#include "mainwindow.h"

#include <QPainter>
#include <QMouseEvent>
#include <QTextEdit>
#include <QHBoxLayout>

#include "lifegamethread.h"
#include "stepthread.h"

PanelWidget::PanelWidget(StepThread *step, QWidget *parent)
    : QWidget(parent),
      step_(step),
      stepCount_(0)
{
}

void PanelWidget::paintEvent(QPaintEvent *){
    QPainter p(this);
    step_->draw(&p);
    p.drawText(740, 120, QString::number(stepCount_));
}


MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent),
      textArea_(new QTextEdit(this)),
      thread_(new LifegameThread(this)),
      step_(new StepThread(this)),
      autoStep_(false),
      timerID_(0),
      stepCount_(0)
{
    initializePixImage();
    step_->setWidth(100);
    step_->setHeight(15);
    step_->setStepSize(1);
    step_->resetField();

    textArea_->setFocusPolicy(Qt::NoFocus);

    QWidget* central = new QWidget(this);
    setCentralWidget(central);
    QBoxLayout* layout = new QBoxLayout(QBoxLayout::TopToBottom, central);
    central->setLayout(layout);

    panel_ = new PanelWidget(step_, central);
    panel_->setMinimumSize(800, 120);
    panel_->setMaximumSize(800, 120);
    layout->addWidget(panel_);
    layout->addWidget(textArea_);
    //layout->addWidget(new QTextEdit(this));

    connect(thread_, SIGNAL(sendMessage(QString)), textArea_, SLOT(append(QString)));
}

MainWindow::~MainWindow()
{

}

void MainWindow::mousePressEvent(QMouseEvent *){
}

void MainWindow::keyPressEvent(QKeyEvent *e){
    if (e->key()==Qt::Key_A){
        if (timerID_ != 0){
            killTimer(timerID_);
            timerID_ = 0;
        }
        else {
            panel_->stepCount_ = 0;
            timerID_ = startTimer(100);
        }
    }
    else if (e->key()==Qt::Key_S){
        if (timerID_ != 0){
            killTimer(timerID_);
            timerID_ = 0;
        }
        ++panel_->stepCount_;
        step_->start();
        step_->wait();
        repaint();
    }
    else if (e->key()==Qt::Key_R){
        step_->resetField();
        panel_->stepCount_ = 0;
        step_->setSeed(thread_->randomPredator().genes_, 1, 0, 0, LifeGame::DNAWidth, LifeGame::DNAHeight);
        step_->setSeed(thread_->randomProducer().genes_, 2, 10, 0, LifeGame::DNAWidth, LifeGame::DNAHeight);
        repaint();
    }
    else if (e->key()==Qt::Key_G){
        if (thread_->isRunning()){
            thread_->stop();
        }
        else {
            thread_->start();
        }
    }
}

void MainWindow::paintEvent(QPaintEvent *){
}

void MainWindow::timerEvent(QTimerEvent *e){
    ++panel_->stepCount_;
    if (panel_->stepCount_ >= 100){
        killTimer(timerID_);
        timerID_ = 0;
    }
    step_->start();
    step_->wait();
    repaint();
}
